@@ -1,3 +1,19 @@
+/*
+ * Copyright 2010-2011 INRIA Saclay
+ * Copyright 2011      Sven Verdoolaege
+ * Copyright 2012-2013 Ecole Normale Superieure
+ *
+ * Use of this software is governed by the MIT license
+ *
+ * Written by Sven Verdoolaege, INRIA Saclay - Ile-de-France,
+ * Parc Club Orsay Universite, ZAC des vignes, 4 rue Jacques Monod,
+ * 91893 Orsay, France
+ * and Ecole Normale Superieure, 45 rue d’Ulm, 75230 Paris, France
+ */
+
+#include <isl/aff.h>
+#include <isl_val_private.h>
+
 #define xFN(TYPE,NAME) TYPE ## _ ## NAME
 #define FN(TYPE,NAME) xFN(TYPE,NAME)
 #define xS(TYPE,NAME) struct TYPE ## _ ## NAME
@@ -154,7 +170,7 @@ __isl_give PW *FN(PW,copy)(__isl_keep PW *pw)
 	return pw;
 }
 
-void *FN(PW,free)(__isl_take PW *pw)
+__isl_null PW *FN(PW,free)(__isl_take PW *pw)
 {
 	int i;
 
@@ -179,12 +195,22 @@ const char *FN(PW,get_dim_name)(__isl_keep PW *pw, enum isl_dim_type type,
 	return pw ? isl_space_get_dim_name(pw->dim, type, pos) : NULL;
 }
 
+int FN(PW,has_dim_id)(__isl_keep PW *pw, enum isl_dim_type type, unsigned pos)
+{
+	return pw ? isl_space_has_dim_id(pw->dim, type, pos) : -1;
+}
+
 __isl_give isl_id *FN(PW,get_dim_id)(__isl_keep PW *pw, enum isl_dim_type type,
 	unsigned pos)
 {
 	return pw ? isl_space_get_dim_id(pw->dim, type, pos) : NULL;
 }
 
+int FN(PW,has_tuple_name)(__isl_keep PW *pw, enum isl_dim_type type)
+{
+	return pw ? isl_space_has_tuple_name(pw->dim, type) : -1;
+}
+
 const char *FN(PW,get_tuple_name)(__isl_keep PW *pw, enum isl_dim_type type)
 {
 	return pw ? isl_space_get_tuple_name(pw->dim, type) : NULL;
@@ -216,7 +242,7 @@ __isl_give PW *FN(PW,realign_domain)(__isl_take PW *pw,
 
 	pw = FN(PW,cow)(pw);
 	if (!pw || !exp)
-		return NULL;
+		goto error;
 
 	for (i = 0; i < pw->n; ++i) {
 		pw->p[i].set = isl_set_realign(pw->p[i].set,
@@ -366,14 +392,14 @@ static __isl_give PW *FN(PW,union_add_aligned)(__isl_take PW *pw1,
 		for (j = 0; j < pw2->n; ++j) {
 			struct isl_set *common;
 			EL *sum;
-			set = isl_set_subtract(set,
-					isl_set_copy(pw2->p[j].set));
 			common = isl_set_intersect(isl_set_copy(pw1->p[i].set),
 						isl_set_copy(pw2->p[j].set));
 			if (isl_set_plain_is_empty(common)) {
 				isl_set_free(common);
 				continue;
 			}
+			set = isl_set_subtract(set,
+					isl_set_copy(pw2->p[j].set));
 
 			sum = FN(EL,add_on_domain)(common,
 						   FN(EL,copy)(pw1->p[i].FIELD),
@@ -506,52 +532,89 @@ __isl_give PW *FN(PW,add_disjoint)(__isl_take PW *pw1, __isl_take PW *pw2)
 
 /* This function is currently only used from isl_aff.c
  */
-static __isl_give PW *FN(PW,on_shared_domain)(__isl_take PW *pw1,
-	__isl_take PW *pw2,
+static __isl_give PW *FN(PW,on_shared_domain_in)(__isl_take PW *pw1,
+	__isl_take PW *pw2, __isl_take isl_space *space,
 	__isl_give EL *(*fn)(__isl_take EL *el1, __isl_take EL *el2))
 	__attribute__ ((unused));
 
 /* Apply "fn" to pairs of elements from pw1 and pw2 on shared domains.
+ * The result of "fn" (and therefore also of this function) lives in "space".
  */
-static __isl_give PW *FN(PW,on_shared_domain)(__isl_take PW *pw1,
-	__isl_take PW *pw2,
+static __isl_give PW *FN(PW,on_shared_domain_in)(__isl_take PW *pw1,
+	__isl_take PW *pw2, __isl_take isl_space *space,
 	__isl_give EL *(*fn)(__isl_take EL *el1, __isl_take EL *el2))
 {
 	int i, j, n;
-	PW *res;
+	PW *res = NULL;
 
 	if (!pw1 || !pw2)
 		goto error;
 
 	n = pw1->n * pw2->n;
 #ifdef HAS_TYPE
-	res = FN(PW,alloc_size)(isl_space_copy(pw1->dim), pw1->type, n);
+	res = FN(PW,alloc_size)(isl_space_copy(space), pw1->type, n);
 #else
-	res = FN(PW,alloc_size)(isl_space_copy(pw1->dim), n);
+	res = FN(PW,alloc_size)(isl_space_copy(space), n);
 #endif
 
 	for (i = 0; i < pw1->n; ++i) {
 		for (j = 0; j < pw2->n; ++j) {
 			isl_set *common;
 			EL *res_ij;
+			int empty;
+
 			common = isl_set_intersect(
 					isl_set_copy(pw1->p[i].set),
 					isl_set_copy(pw2->p[j].set));
-			if (isl_set_plain_is_empty(common)) {
+			empty = isl_set_plain_is_empty(common);
+			if (empty < 0 || empty) {
 				isl_set_free(common);
+				if (empty < 0)
+					goto error;
 				continue;
 			}
 
 			res_ij = fn(FN(EL,copy)(pw1->p[i].FIELD),
 				    FN(EL,copy)(pw2->p[j].FIELD));
+			res_ij = FN(EL,gist)(res_ij, isl_set_copy(common));
 
 			res = FN(PW,add_piece)(res, common, res_ij);
 		}
 	}
 
+	isl_space_free(space);
 	FN(PW,free)(pw1);
 	FN(PW,free)(pw2);
 	return res;
+error:
+	isl_space_free(space);
+	FN(PW,free)(pw1);
+	FN(PW,free)(pw2);
+	FN(PW,free)(res);
+	return NULL;
+}
+
+/* This function is currently only used from isl_aff.c
+ */
+static __isl_give PW *FN(PW,on_shared_domain)(__isl_take PW *pw1,
+	__isl_take PW *pw2,
+	__isl_give EL *(*fn)(__isl_take EL *el1, __isl_take EL *el2))
+	__attribute__ ((unused));
+
+/* Apply "fn" to pairs of elements from pw1 and pw2 on shared domains.
+ * The result of "fn" is assumed to live in the same space as "pw1" and "pw2".
+ */
+static __isl_give PW *FN(PW,on_shared_domain)(__isl_take PW *pw1,
+	__isl_take PW *pw2,
+	__isl_give EL *(*fn)(__isl_take EL *el1, __isl_take EL *el2))
+{
+	isl_space *space;
+
+	if (!pw1 || !pw2)
+		goto error;
+
+	space = isl_space_copy(pw1->dim);
+	return FN(PW,on_shared_domain_in)(pw1, pw2, space, fn);
 error:
 	FN(PW,free)(pw1);
 	FN(PW,free)(pw2);
@@ -589,20 +652,20 @@ __isl_give PW *FN(PW,sub)(__isl_take PW *pw1, __isl_take PW *pw2)
 #endif
 
 #ifndef NO_EVAL
-__isl_give isl_qpolynomial *FN(PW,eval)(__isl_take PW *pw,
-	__isl_take isl_point *pnt)
+__isl_give isl_val *FN(PW,eval)(__isl_take PW *pw, __isl_take isl_point *pnt)
 {
 	int i;
 	int found = 0;
 	isl_ctx *ctx;
 	isl_space *pnt_dim = NULL;
-	isl_qpolynomial *qp;
+	isl_val *v;
 
 	if (!pw || !pnt)
 		goto error;
 	ctx = isl_point_get_ctx(pnt);
 	pnt_dim = isl_point_get_space(pnt);
-	isl_assert(ctx, isl_space_is_domain(pnt_dim, pw->dim), goto error);
+	isl_assert(ctx, isl_space_is_domain_internal(pnt_dim, pw->dim),
+		    goto error);
 
 	for (i = 0; i < pw->n; ++i) {
 		found = isl_set_contains_point(pw->p[i].set, pnt);
@@ -612,14 +675,14 @@ __isl_give isl_qpolynomial *FN(PW,eval)(__isl_take PW *pw,
 			break;
 	}
 	if (found)
-		qp = FN(EL,eval)(FN(EL,copy)(pw->p[i].FIELD),
+		v = FN(EL,eval)(FN(EL,copy)(pw->p[i].FIELD),
 					    isl_point_copy(pnt));
 	else
-		qp = isl_qpolynomial_zero_on_domain(FN(PW,get_domain_space)(pw));
+		v = isl_val_zero(ctx);
 	FN(PW,free)(pw);
 	isl_space_free(pnt_dim);
 	isl_point_free(pnt);
-	return qp;
+	return v;
 error:
 	FN(PW,free)(pw);
 	isl_space_free(pnt_dim);
@@ -628,6 +691,13 @@ error:
 }
 #endif
 
+/* Return the parameter domain of "pw".
+ */
+__isl_give isl_set *FN(PW,params)(__isl_take PW *pw)
+{
+	return isl_set_params(FN(PW,domain)(pw));
+}
+
 __isl_give isl_set *FN(PW,domain)(__isl_take PW *pw)
 {
 	int i;
@@ -645,6 +715,91 @@ __isl_give isl_set *FN(PW,domain)(__isl_take PW *pw)
 	return dom;
 }
 
+/* Exploit the equalities in the domain of piece "i" of "pw"
+ * to simplify the associated function.
+ * If the domain of piece "i" is empty, then remove it entirely,
+ * replacing it with the final piece.
+ */
+static int FN(PW,exploit_equalities_and_remove_if_empty)(__isl_keep PW *pw,
+	int i)
+{
+	isl_basic_set *aff;
+	int empty = isl_set_plain_is_empty(pw->p[i].set);
+
+	if (empty < 0)
+		return -1;
+	if (empty) {
+		isl_set_free(pw->p[i].set);
+		FN(EL,free)(pw->p[i].FIELD);
+		if (i != pw->n - 1)
+			pw->p[i] = pw->p[pw->n - 1];
+		pw->n--;
+
+		return 0;
+	}
+
+	aff = isl_set_affine_hull(isl_set_copy(pw->p[i].set));
+	pw->p[i].FIELD = FN(EL,substitute_equalities)(pw->p[i].FIELD, aff);
+	if (!pw->p[i].FIELD)
+		return -1;
+
+	return 0;
+}
+
+/* Convert a piecewise expression defined over a parameter domain
+ * into one that is defined over a zero-dimensional set.
+ */
+__isl_give PW *FN(PW,from_range)(__isl_take PW *pw)
+{
+	isl_space *space;
+
+	if (!pw)
+		return NULL;
+	if (!isl_space_is_set(pw->dim))
+		isl_die(FN(PW,get_ctx)(pw), isl_error_invalid,
+			"not living in a set space", return FN(PW,free)(pw));
+
+	space = FN(PW,get_space)(pw);
+	space = isl_space_from_range(space);
+	pw = FN(PW,reset_space)(pw, space);
+
+	return pw;
+}
+
+/* Fix the value of the given parameter or domain dimension of "pw"
+ * to be equal to "value".
+ */
+__isl_give PW *FN(PW,fix_si)(__isl_take PW *pw, enum isl_dim_type type,
+	unsigned pos, int value)
+{
+	int i;
+
+	if (!pw)
+		return NULL;
+
+	if (type == isl_dim_out)
+		isl_die(FN(PW,get_ctx)(pw), isl_error_invalid,
+			"cannot fix output dimension", return FN(PW,free)(pw));
+
+	if (pw->n == 0)
+		return pw;
+
+	if (type == isl_dim_in)
+		type = isl_dim_set;
+
+	pw = FN(PW,cow)(pw);
+	if (!pw)
+		return FN(PW,free)(pw);
+
+	for (i = pw->n - 1; i >= 0; --i) {
+		pw->p[i].set = isl_set_fix_si(pw->p[i].set, type, pos, value);
+		if (FN(PW,exploit_equalities_and_remove_if_empty)(pw, i) < 0)
+			return FN(PW,free)(pw);
+	}
+
+	return pw;
+}
+
 /* Restrict the domain of "pw" by combining each cell
  * with "set" through a call to "fn", where "fn" may be
  * isl_set_intersect or isl_set_intersect_params.
@@ -669,22 +824,9 @@ static __isl_give PW *FN(PW,intersect_aligned)(__isl_take PW *pw,
 		goto error;
 
 	for (i = pw->n - 1; i >= 0; --i) {
-		isl_basic_set *aff;
 		pw->p[i].set = fn(pw->p[i].set, isl_set_copy(set));
-		if (!pw->p[i].set)
+		if (FN(PW,exploit_equalities_and_remove_if_empty)(pw, i) < 0)
 			goto error;
-		aff = isl_set_affine_hull(isl_set_copy(pw->p[i].set));
-		pw->p[i].FIELD = FN(EL,substitute_equalities)(pw->p[i].FIELD,
-								aff);
-		if (!pw->p[i].FIELD)
-			goto error;
-		if (isl_set_plain_is_empty(pw->p[i].set)) {
-			isl_set_free(pw->p[i].set);
-			FN(EL,free)(pw->p[i].FIELD);
-			if (i != pw->n - 1)
-				pw->p[i] = pw->p[pw->n - 1];
-			pw->n--;
-		}
 	}
 	
 	isl_set_free(set);
@@ -755,16 +897,17 @@ static __isl_give PW *FN(PW,gist_aligned)(__isl_take PW *pw,
 		goto error;
 
 	for (i = pw->n - 1; i >= 0; --i) {
-		pw->p[i].set = isl_set_intersect(pw->p[i].set,
+		isl_set *set_i;
+		int empty;
+
+		set_i = isl_set_intersect(isl_set_copy(pw->p[i].set),
 						 isl_set_copy(context));
-		if (!pw->p[i].set)
-			goto error;
-		pw->p[i].FIELD = fn_el(pw->p[i].FIELD,
-					     isl_set_copy(pw->p[i].set));
+		empty = isl_set_plain_is_empty(set_i);
+		pw->p[i].FIELD = fn_el(pw->p[i].FIELD, set_i);
 		pw->p[i].set = fn_dom(pw->p[i].set, isl_basic_set_copy(hull));
-		if (!pw->p[i].set)
+		if (!pw->p[i].FIELD || !pw->p[i].set)
 			goto error;
-		if (isl_set_plain_is_empty(pw->p[i].set)) {
+		if (empty) {
 			isl_set_free(pw->p[i].set);
 			FN(EL,free)(pw->p[i].FIELD);
 			if (i != pw->n - 1)
@@ -932,12 +1075,14 @@ __isl_give PW *FN(PW,drop_dims)(__isl_take PW *pw,
 	if (!pw->dim)
 		goto error;
 	for (i = 0; i < pw->n; ++i) {
-		pw->p[i].set = isl_set_drop(pw->p[i].set, set_type, first, n);
-		if (!pw->p[i].set)
-			goto error;
 		pw->p[i].FIELD = FN(EL,drop_dims)(pw->p[i].FIELD, type, first, n);
 		if (!pw->p[i].FIELD)
 			goto error;
+		if (type == isl_dim_out)
+			continue;
+		pw->p[i].set = isl_set_drop(pw->p[i].set, set_type, first, n);
+		if (!pw->p[i].set)
+			goto error;
 	}
 
 	return pw;
@@ -1058,14 +1203,32 @@ __isl_give PW *FN(PW,fix_dim)(__isl_take PW *pw,
 		return NULL;
 	for (i = 0; i < pw->n; ++i) {
 		pw->p[i].set = isl_set_fix(pw->p[i].set, type, pos, v);
-		if (!pw->p[i].set)
-			goto error;
+		if (FN(PW,exploit_equalities_and_remove_if_empty)(pw, i) < 0)
+			return FN(PW,free)(pw);
 	}
 
+	return pw;
+}
+
+/* Fix the value of the variable at position "pos" of type "type" of "pw"
+ * to be equal to "v".
+ */
+__isl_give PW *FN(PW,fix_val)(__isl_take PW *pw,
+	enum isl_dim_type type, unsigned pos, __isl_take isl_val *v)
+{
+	if (!v)
+		return FN(PW,free)(pw);
+	if (!isl_val_is_int(v))
+		isl_die(FN(PW,get_ctx)(pw), isl_error_invalid,
+			"expecting integer value", goto error);
+
+	pw = FN(PW,fix_dim)(pw, type, pos, v->n);
+	isl_val_free(v);
+
 	return pw;
 error:
-	FN(PW,free)(pw);
-	return NULL;
+	isl_val_free(v);
+	return FN(PW,free)(pw);
 }
 
 unsigned FN(PW,dim)(__isl_keep PW *pw, enum isl_dim_type type)
@@ -1109,42 +1272,42 @@ error:
  * In the worst case, the domain is scanned completely,
  * so the domain is assumed to be bounded.
  */
-__isl_give isl_qpolynomial *FN(PW,opt)(__isl_take PW *pw, int max)
+__isl_give isl_val *FN(PW,opt)(__isl_take PW *pw, int max)
 {
 	int i;
-	isl_qpolynomial *opt;
+	isl_val *opt;
 
 	if (!pw)
 		return NULL;
 
 	if (pw->n == 0) {
-		isl_space *dim = isl_space_copy(pw->dim);
+		opt = isl_val_zero(FN(PW,get_ctx)(pw));
 		FN(PW,free)(pw);
-		return isl_qpolynomial_zero_on_domain(isl_space_domain(dim));
+		return opt;
 	}
 
 	opt = FN(EL,opt_on_domain)(FN(EL,copy)(pw->p[0].FIELD),
 					isl_set_copy(pw->p[0].set), max);
 	for (i = 1; i < pw->n; ++i) {
-		isl_qpolynomial *opt_i;
+		isl_val *opt_i;
 		opt_i = FN(EL,opt_on_domain)(FN(EL,copy)(pw->p[i].FIELD),
 						isl_set_copy(pw->p[i].set), max);
 		if (max)
-			opt = isl_qpolynomial_max_cst(opt, opt_i);
+			opt = isl_val_max(opt, opt_i);
 		else
-			opt = isl_qpolynomial_min_cst(opt, opt_i);
+			opt = isl_val_min(opt, opt_i);
 	}
 
 	FN(PW,free)(pw);
 	return opt;
 }
 
-__isl_give isl_qpolynomial *FN(PW,max)(__isl_take PW *pw)
+__isl_give isl_val *FN(PW,max)(__isl_take PW *pw)
 {
 	return FN(PW,opt)(pw, 1);
 }
 
-__isl_give isl_qpolynomial *FN(PW,min)(__isl_take PW *pw)
+__isl_give isl_val *FN(PW,min)(__isl_take PW *pw)
 {
 	return FN(PW,opt)(pw, 0);
 }
@@ -1216,18 +1379,42 @@ __isl_give PW *FN(PW,reset_space)(__isl_take PW *pw, __isl_take isl_space *dim)
 	return FN(PW,reset_space_and_domain)(pw, dim, domain);
 }
 
-__isl_give PW *FN(PW,set_tuple_id)(__isl_keep PW *pw, enum isl_dim_type type,
+__isl_give PW *FN(PW,set_tuple_id)(__isl_take PW *pw, enum isl_dim_type type,
 	__isl_take isl_id *id)
 {
 	isl_space *space;
 
 	pw = FN(PW,cow)(pw);
 	if (!pw)
-		return isl_id_free(id);
+		goto error;
 
 	space = FN(PW,get_space)(pw);
 	space = isl_space_set_tuple_id(space, type, id);
 
+	return FN(PW,reset_space)(pw, space);
+error:
+	isl_id_free(id);
+	return FN(PW,free)(pw);
+}
+
+/* Drop the id on the specified tuple.
+ */
+__isl_give PW *FN(PW,reset_tuple_id)(__isl_take PW *pw, enum isl_dim_type type)
+{
+	isl_space *space;
+
+	if (!pw)
+		return NULL;
+	if (!FN(PW,has_tuple_id)(pw, type))
+		return pw;
+
+	pw = FN(PW,cow)(pw);
+	if (!pw)
+		return NULL;
+
+	space = FN(PW,get_space)(pw);
+	space = isl_space_reset_tuple_id(space, type);
+
 	return FN(PW,reset_space)(pw, space);
 }
 
@@ -1236,9 +1423,12 @@ __isl_give PW *FN(PW,set_dim_id)(__isl_take PW *pw,
 {
 	pw = FN(PW,cow)(pw);
 	if (!pw)
-		return isl_id_free(id);
+		goto error;
 	pw->dim = isl_space_set_dim_id(pw->dim, type, pos, id);
 	return FN(PW,reset_space)(pw, isl_space_copy(pw->dim));
+error:
+	isl_id_free(id);
+	return FN(PW,free)(pw);
 }
 #endif
 
@@ -1261,7 +1451,7 @@ __isl_give PW *FN(PW,morph_domain)(__isl_take PW *pw,
 		goto error;
 
 	ctx = isl_space_get_ctx(pw->dim);
-	isl_assert(ctx, isl_space_is_domain(morph->dom->dim, pw->dim),
+	isl_assert(ctx, isl_space_is_domain_internal(morph->dom->dim, pw->dim),
 		goto error);
 
 	pw = FN(PW,cow)(pw);
@@ -1292,6 +1482,11 @@ error:
 }
 #endif
 
+int FN(PW,n_piece)(__isl_keep PW *pw)
+{
+	return pw ? pw->n : 0;
+}
+
 int FN(PW,foreach_piece)(__isl_keep PW *pw,
 	int (*fn)(__isl_take isl_set *set, __isl_take EL *el, void *user),
 	void *user)
@@ -1433,7 +1628,7 @@ __isl_give PW *FN(PW,mul_isl_int)(__isl_take PW *pw, isl_int v)
 
 	if (isl_int_is_one(v))
 		return pw;
-	if (pw && isl_int_is_zero(v)) {
+	if (pw && DEFAULT_IS_ZERO && isl_int_is_zero(v)) {
 		PW *zero;
 		isl_space *dim = FN(PW,get_space)(pw);
 #ifdef HAS_TYPE
@@ -1466,6 +1661,58 @@ error:
 	return NULL;
 }
 
+/* Multiply the pieces of "pw" by "v" and return the result.
+ */
+__isl_give PW *FN(PW,scale_val)(__isl_take PW *pw, __isl_take isl_val *v)
+{
+	int i;
+
+	if (!pw || !v)
+		goto error;
+
+	if (isl_val_is_one(v)) {
+		isl_val_free(v);
+		return pw;
+	}
+	if (pw && DEFAULT_IS_ZERO && isl_val_is_zero(v)) {
+		PW *zero;
+		isl_space *space = FN(PW,get_space)(pw);
+#ifdef HAS_TYPE
+		zero = FN(PW,ZERO)(space, pw->type);
+#else
+		zero = FN(PW,ZERO)(space);
+#endif
+		FN(PW,free)(pw);
+		isl_val_free(v);
+		return zero;
+	}
+	if (pw->n == 0) {
+		isl_val_free(v);
+		return pw;
+	}
+	pw = FN(PW,cow)(pw);
+	if (!pw)
+		goto error;
+
+#ifdef HAS_TYPE
+	if (isl_val_is_neg(v))
+		pw->type = isl_fold_type_negate(pw->type);
+#endif
+	for (i = 0; i < pw->n; ++i) {
+		pw->p[i].FIELD = FN(EL,scale_val)(pw->p[i].FIELD,
+						    isl_val_copy(v));
+		if (!pw->p[i].FIELD)
+			goto error;
+	}
+
+	isl_val_free(v);
+	return pw;
+error:
+	isl_val_free(v);
+	FN(PW,free)(pw);
+	return NULL;
+}
+
 __isl_give PW *FN(PW,scale)(__isl_take PW *pw, isl_int v)
 {
 	return FN(PW,mul_isl_int)(pw, v);
@@ -1473,8 +1720,8 @@ __isl_give PW *FN(PW,scale)(__isl_take PW *pw, isl_int v)
 
 static int FN(PW,qsort_set_cmp)(const void *p1, const void *p2)
 {
-	const isl_set *set1 = *(const isl_set **)p1;
-	const isl_set *set2 = *(const isl_set **)p2;
+	isl_set *set1 = *(isl_set * const *)p1;
+	isl_set *set2 = *(isl_set * const *)p2;
 
 	return isl_set_plain_cmp(set1, set2);
 }
@@ -1563,3 +1810,161 @@ error:
 	FN(PW,free)(pw2);
 	return -1;
 }
+
+#ifndef NO_PULLBACK
+static __isl_give PW *FN(PW,align_params_pw_multi_aff_and)(__isl_take PW *pw,
+	__isl_take isl_multi_aff *ma,
+	__isl_give PW *(*fn)(__isl_take PW *pw, __isl_take isl_multi_aff *ma))
+{
+	isl_ctx *ctx;
+	isl_space *ma_space;
+
+	ma_space = isl_multi_aff_get_space(ma);
+	if (!pw || !ma || !ma_space)
+		goto error;
+	if (isl_space_match(pw->dim, isl_dim_param, ma_space, isl_dim_param)) {
+		isl_space_free(ma_space);
+		return fn(pw, ma);
+	}
+	ctx = FN(PW,get_ctx)(pw);
+	if (!isl_space_has_named_params(pw->dim) ||
+	    !isl_space_has_named_params(ma_space))
+		isl_die(ctx, isl_error_invalid,
+			"unaligned unnamed parameters", goto error);
+	pw = FN(PW,align_params)(pw, ma_space);
+	ma = isl_multi_aff_align_params(ma, FN(PW,get_space)(pw));
+	return fn(pw, ma);
+error:
+	isl_space_free(ma_space);
+	FN(PW,free)(pw);
+	isl_multi_aff_free(ma);
+	return NULL;
+}
+
+static __isl_give PW *FN(PW,align_params_pw_pw_multi_aff_and)(__isl_take PW *pw,
+	__isl_take isl_pw_multi_aff *pma,
+	__isl_give PW *(*fn)(__isl_take PW *pw,
+		__isl_take isl_pw_multi_aff *ma))
+{
+	isl_ctx *ctx;
+	isl_space *pma_space;
+
+	pma_space = isl_pw_multi_aff_get_space(pma);
+	if (!pw || !pma || !pma_space)
+		goto error;
+	if (isl_space_match(pw->dim, isl_dim_param, pma_space, isl_dim_param)) {
+		isl_space_free(pma_space);
+		return fn(pw, pma);
+	}
+	ctx = FN(PW,get_ctx)(pw);
+	if (!isl_space_has_named_params(pw->dim) ||
+	    !isl_space_has_named_params(pma_space))
+		isl_die(ctx, isl_error_invalid,
+			"unaligned unnamed parameters", goto error);
+	pw = FN(PW,align_params)(pw, pma_space);
+	pma = isl_pw_multi_aff_align_params(pma, FN(PW,get_space)(pw));
+	return fn(pw, pma);
+error:
+	isl_space_free(pma_space);
+	FN(PW,free)(pw);
+	isl_pw_multi_aff_free(pma);
+	return NULL;
+}
+
+/* Compute the pullback of "pw" by the function represented by "ma".
+ * In other words, plug in "ma" in "pw".
+ */
+static __isl_give PW *FN(PW,pullback_multi_aff_aligned)(__isl_take PW *pw,
+	__isl_take isl_multi_aff *ma)
+{
+	int i;
+	isl_space *space = NULL;
+
+	ma = isl_multi_aff_align_divs(ma);
+	pw = FN(PW,cow)(pw);
+	if (!pw || !ma)
+		goto error;
+
+	space = isl_space_join(isl_multi_aff_get_space(ma),
+				FN(PW,get_space)(pw));
+
+	for (i = 0; i < pw->n; ++i) {
+		pw->p[i].set = isl_set_preimage_multi_aff(pw->p[i].set,
+						    isl_multi_aff_copy(ma));
+		if (!pw->p[i].set)
+			goto error;
+		pw->p[i].FIELD = FN(EL,pullback_multi_aff)(pw->p[i].FIELD,
+						    isl_multi_aff_copy(ma));
+		if (!pw->p[i].FIELD)
+			goto error;
+	}
+
+	pw = FN(PW,reset_space)(pw, space);
+	isl_multi_aff_free(ma);
+	return pw;
+error:
+	isl_space_free(space);
+	isl_multi_aff_free(ma);
+	FN(PW,free)(pw);
+	return NULL;
+}
+
+__isl_give PW *FN(PW,pullback_multi_aff)(__isl_take PW *pw,
+	__isl_take isl_multi_aff *ma)
+{
+	return FN(PW,align_params_pw_multi_aff_and)(pw, ma,
+					&FN(PW,pullback_multi_aff_aligned));
+}
+
+/* Compute the pullback of "pw" by the function represented by "pma".
+ * In other words, plug in "pma" in "pw".
+ */
+static __isl_give PW *FN(PW,pullback_pw_multi_aff_aligned)(__isl_take PW *pw,
+	__isl_take isl_pw_multi_aff *pma)
+{
+	int i;
+	PW *res;
+
+	if (!pma)
+		goto error;
+
+	if (pma->n == 0) {
+		isl_space *space;
+		space = isl_space_join(isl_pw_multi_aff_get_space(pma),
+					FN(PW,get_space)(pw));
+		isl_pw_multi_aff_free(pma);
+		res = FN(PW,empty)(space);
+		FN(PW,free)(pw);
+		return res;
+	}
+
+	res = FN(PW,pullback_multi_aff)(FN(PW,copy)(pw),
+					isl_multi_aff_copy(pma->p[0].maff));
+	res = FN(PW,intersect_domain)(res, isl_set_copy(pma->p[0].set));
+
+	for (i = 1; i < pma->n; ++i) {
+		PW *res_i;
+
+		res_i = FN(PW,pullback_multi_aff)(FN(PW,copy)(pw),
+					isl_multi_aff_copy(pma->p[i].maff));
+		res_i = FN(PW,intersect_domain)(res_i,
+					isl_set_copy(pma->p[i].set));
+		res = FN(PW,add_disjoint)(res, res_i);
+	}
+
+	isl_pw_multi_aff_free(pma);
+	FN(PW,free)(pw);
+	return res;
+error:
+	isl_pw_multi_aff_free(pma);
+	FN(PW,free)(pw);
+	return NULL;
+}
+
+__isl_give PW *FN(PW,pullback_pw_multi_aff)(__isl_take PW *pw,
+	__isl_take isl_pw_multi_aff *pma)
+{
+	return FN(PW,align_params_pw_pw_multi_aff_and)(pw, pma,
+					&FN(PW,pullback_pw_multi_aff_aligned));
+}
+#endif
