@@ -144,7 +144,7 @@ def _add_functionality():
         """
         result = {}
 
-        def set_dim_name(name, tp, idx):
+        def set_dim_id(name, tp, idx):
             if name in result:
                 raise RuntimeError("non-unique var id '%s' encountered" % name)
             result[name] = tp, idx
@@ -294,7 +294,7 @@ def _add_functionality():
         result = {}
         for tp in types:
             for i in range(self.get_space().dim(tp)):
-                coeff = self.get_coefficient(tp, i)
+                coeff = self.get_coefficient_val(tp, i)
                 if coeff:
                     if dim_to_name is None:
                         name = self.get_dim_name(tp, i)
@@ -303,7 +303,7 @@ def _add_functionality():
 
                     result[name] = coeff
 
-        const = self.get_constant()
+        const = self.get_constant_val()
         if const:
             result[1] = const
 
@@ -553,9 +553,17 @@ def _add_functionality():
         if context is None:
             context = _DEFAULT_CONTEXT
 
-        if isinstance(src, (str, unicode)):
+        import sys
+        if sys.version_info >= (3,):
+            string_types = str
+            int_types = int
+        else:
+            string_types = basestring
+            int_types = (int, long)
+
+        if isinstance(src, string_types):
             result = cls.read_from_str(context, src)
-        elif isinstance(src, (int, long)):
+        elif isinstance(src, int_types):
             result = cls.int_from_si(context, src)
         else:
             raise TypeError("'src' must be int or string")
@@ -580,7 +588,11 @@ def _add_functionality():
         if not self.is_int():
             raise ValueError("can only convert integer Val to python")
 
-        return long(self.to_str())
+        import sys
+        if sys.version_info >= (3,):
+            return int(self.to_str())
+        else:
+            return long(self.to_str())
 
     Val.__new__ = staticmethod(val_new)
     Val.__init__ = val_bogus_init
